﻿using Kivii.DataAnnotations;
using Kivii.Finances.Entities;
using Kivii.Finances.Seeyon.Extensions;
using Kivii.Linq;
using Kivii.Seeyon;
using Kivii.Seeyon.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Seeyon.Entities
{
    [Api(Description = "账单申请")]
    [Alias(Configs.TableNameApply)]
    internal class BillApply : InvoiceApply, IEntityIsSeeyonApply
    {
        public object OnGetApplyResult(Guid Kvid, object Result, string Message)
        {
            if (Kvid == Guid.Empty) throw new Exception("缺少业务编号!");
            if (Result == null) throw new Exception("缺少审批结果信息");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var rtns = new RestfulExecutionResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();
            var apply = conn.SingleById<InvoiceApply>(Kvid);
            if (apply == null) throw new Exception("未找到此业务信息!");
            var result = Result.ToString();
            if (apply.Metadata.IsNullOrEmpty()) apply.Metadata = new Dictionary<string, string>();
            EntityLog<InvoiceApply> log = null;
            if (result == "审批通过")
            {
                apply.Metadata["SeeyonResult"] = result;
                apply.AddOnlyProperties(o => o.Metadata);
                if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                {
                    apply.Status = (int)InvoiceApplyStatus.FinancialApproval;
                    apply.AddOnlyProperties(o => o.Status);
                    log = new EntityLog<InvoiceApply>();
                    log.OwnerKvid = apply.Kvid;
                    log.Title = "账单借票";
                    log.Type = "流转";
                    log.Summary = "OA审批通过,可进行开票";
                    log.Remark = Message;
                    log.CreatorName = "OA审批";
                }
            }
            else if (result == "审批不通过")
            {
                apply.Metadata["SeeyonResult"] = result;
                apply.Metadata["SeeyonMessage"] = Message;
                apply.AddOnlyProperties(o => o.Metadata);
                if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                {
                    apply.Status = (int)InvoiceApplyStatus.ApplyReject;
                    apply.AddOnlyProperties(o => o.Status);
                    log = new EntityLog<InvoiceApply>();
                    log.OwnerKvid = apply.Kvid;
                    log.Title = "账单借票";
                    log.Type = "流转";
                    log.Summary = "OA审批不通过,驳回了账单借票审批";
                    log.Remark = Message;
                    log.CreatorName = "OA审批";
                }
            }
            if (log != null) conn.Insert(log);
            if (!apply.OnlyProperties.IsNullOrEmpty())
            {
                conn.UpdateOnly(apply);
                rtns.Results.Add(apply);
            }

            return rtns;
        }

        public object OnGetApplyResult(List<Guid> Kvids, object Result, string Message)
        {
            if (Kvids.IsNullOrEmpty()) throw new Exception("缺少业务编号!");
            if (Result == null) throw new Exception("缺少审批结果信息");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var rtns = new RestfulExecutionResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();
            var applys = conn.SelectByIds<InvoiceApply>(Kvids);
            if (applys.IsNullOrEmpty()) throw new Exception("未找到此业务信息!");
            //var apply = conn.SingleById<InvoiceApply>(Kvids);
            //if (apply == null) throw new Exception("未找到此业务信息!");
            var result = Result.ToString();
            foreach (var apply in applys)
            {
                if (apply.Metadata.IsNullOrEmpty()) apply.Metadata = new Dictionary<string, string>();
                EntityLog<InvoiceApply> log = null;
                if (result == "审批通过")
                {
                    apply.Metadata["SeeyonResult"] = result;
                    apply.AddOnlyProperties(o => o.Metadata);
                    if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                    {
                        apply.Status = (int)InvoiceApplyStatus.FinancialApproval;
                        apply.AddOnlyProperties(o => o.Status);
                        log = new EntityLog<InvoiceApply>();
                        log.OwnerKvid = apply.Kvid;
                        log.Title = "账单借票";
                        log.Type = "流转";
                        log.Summary = "OA审批通过,可进行开票";
                        log.Remark = Message;
                        log.CreatorName = "OA审批";
                    }
                }
                else if (result == "审批不通过")
                {
                    apply.Metadata["SeeyonResult"] = result;
                    apply.Metadata["SeeyonMessage"] = Message;
                    apply.AddOnlyProperties(o => o.Metadata);
                    if (apply.Status == (int)InvoiceApplyStatus.ProcessAdoption)
                    {
                        apply.Status = (int)InvoiceApplyStatus.ApplyReject;
                        apply.AddOnlyProperties(o => o.Status);
                        log = new EntityLog<InvoiceApply>();
                        log.OwnerKvid = apply.Kvid;
                        log.Title = "账单借票";
                        log.Type = "流转";
                        log.Summary = "OA审批不通过,驳回了账单借票审批";
                        log.Remark = Message;
                        log.CreatorName = "OA审批";
                    }
                }
                if (log != null) conn.Insert(log);
                if (!apply.OnlyProperties.IsNullOrEmpty())
                {
                    conn.UpdateOnly(apply);
                    rtns.Results.Add(apply);
                }
            }
            return rtns;
        }

        public object OnSubmitApplyForm<T>(Guid Kvid)
        {
            Kvid.ThrowIfEmpty("请传入要申请的账单借票信息!");
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            InvoiceApply apply = conn.SingleById<InvoiceApply>(Kvid);
            apply.ThrowIfNull("未找到此业务信息!");
            if (apply.Status > (int)InvoiceApplyStatus.ProcessAdoption) throw new Exception("当前账单借票非待审批状态，无法申请审批！");
            var applyForm = apply.ConvertToForm();
            var applyFormDetail = apply.ConvertToFormDetail();
            var session = KiviiContext.Request.GetSession();

            var client = new JsonServiceClient(Configs.BaseUrl);
            var resp = client.RequestBillForm(applyForm, applyFormDetail, session);
            if (resp.code != "0") throw new Exception($"OA接口调用失败,原因:{resp.message}");
            apply.BizId = resp.data.subject;
            apply.AddOnlyProperties(o => o.BizId);
            apply.BizType = typeof(BillApply).FullName;
            apply.AddOnlyProperties(o => o.BizType);
            apply.Status = (int)InvoiceApplyStatus.ProcessAdoption;
            apply.AddOnlyProperties(o => o.Status);
            conn.UpdateOnly(apply);
            var _log = new EntityLog<InvoiceApply>();
            _log.OwnerKvid = apply.Kvid;
            _log.Title = "账单借票";
            _log.Type = "流转";
            _log.Summary = $"提交了账单借票,审批将在OA执行";
            conn.Insert(_log);
            var rtns = new RestfulUpdateResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();
            rtns.Results.Add(apply);

            return rtns;
        }

        public object OnSubmitApplyForm<T>(T form)
        {
            form.ThrowIfNull("请传入借票信息！");
            var formApply = form as BillApply;
            formApply.ThrowIfNull("传入的借票信息不能为空！");
            var kvids = formApply.Kvids;
            var conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var applys = conn.SelectByIds<InvoiceApply>(kvids);
            applys.ThrowIfNullOrEmpty("未找到借票信息!");
            if (applys.Exists(o => o.Status > (int)InvoiceApplyStatus.ProcessAdoption)) throw new Exception("当前账单借票非待审批状态，无法申请审批！");

            //var groupPayer = applys.GroupBy(a => a.PayerName);
            //if (groupPayer.Count() != 1) throw new Exception("不允许多个借票抬头同时申请！");
            //var applySum = new InvoiceApply();
            //applySum.PopulateWith(applys[0]);
            //applySum.Amount = applys.Sum(a => a.Amount);
            //applySum.Remark = applys[0].Remark;

            var applyForm = applys.ConvertToForm();
            var applyFormDetail = applys.ConvertToFormDetail();

            //var bizKvids = applys.ConvertAll(o => o.Kvid);
            //var strBizKvids = string.Join(",", bizKvids);
            //applyForm.业务编号 = strBizKvids;

            var session = KiviiContext.Request.GetSession();

            var client = new JsonServiceClient(Configs.BaseUrl);
            var resp = client.RequestBillForm(applyForm, applyFormDetail, session);
            if (resp.code != "0") throw new Exception($"OA接口调用失败,原因:{resp.message}");
            var rtns = new RestfulUpdateResponse<InvoiceApply>();
            rtns.Results = new List<InvoiceApply>();
            foreach (var apply in applys)
            {
                apply.BizId = resp.data.subject;
                apply.AddOnlyProperties(o => o.BizId);
                apply.BizType = typeof(BillApply).FullName;
                apply.AddOnlyProperties(o => o.BizType);
                apply.Status = (int)InvoiceApplyStatus.ProcessAdoption;
                apply.AddOnlyProperties(o => o.Status);
                conn.UpdateOnly(apply);
                var _log = new EntityLog<InvoiceApply>();
                _log.OwnerKvid = apply.Kvid;
                _log.Title = "账单借票";
                _log.Type = "流转";
                _log.Summary = $"提交了账单借票,审批将在OA执行";
                conn.Insert(_log);

                rtns.Results.Add(apply);
            }


            return rtns;
        }

        [Ignore]
        public List<Guid> Kvids { get; set; }
    }

    public class RequestForm_InvoiceApply
    {
        public Form_InvoiceApply formmain_0171 { get; set; }
        public List<Form_InvoiceApplyDetail> formson_0172 { get; set; }
    }
    public class Form_InvoiceApply
    {
        /// <summary>
        /// 唯一编号,BizKvid
        /// </summary>
        public string 业务编号 { get; set; }

        /// <summary>
        /// 业务类型,BizType
        /// </summary>
        public string 业务类型 { get; set; }
        /// <summary>
        /// 单据编号
        /// </summary>
        public string 编号 { get; set; }

        public string 借票人 { get; set; }

        public string 借票科室 { get; set; }

        /// <summary>
        /// 格式：2023-10-11
        /// </summary>
        public string 借票日期 { get; set; }

        public string 借票总额 { get; set; }

        /// <summary>
        /// 金额大写
        /// </summary>
        public string 借票总额人民币 { get; set; }


        //public string 付款单位名称 { get; set; }
        //public string 纳税人识别号 { get; set; }
        //public string 地址电话 { get; set; }
        //public string 开户行及账号 { get; set; }

        ///// <summary>
        ///// 发票类型{普通：4529935671729733794  专票：-4189929216282050282}
        ///// </summary>
        //public string 发票类型 { get; set; }
        //public string 申请原因 { get; set; }
        //public string 数量 { get; set; }
    }

    public class Form_InvoiceApplyDetail
    {
        public string 序号1 { get; set; }
        public string 单位名称 { get; set; }

        public string 金额 { get; set; }
    }
}
